/** Concurrent lexical analysis [Andrews-91,p.165]
 *
 * Consider a simple language for expression evaluation :
 *  expr ::= term | expression op term
 *  term ::= id | number
 *    op ::= + | -
 *
 * An identifier is a sequence of letters and digits, beginning by a letter.
 * A number is a sequence of digits.
 *
 * Given is an array of characters ch[1:n]. Each character is either a letter,
 * a digit, a blank, +, or *. The sequence of characters from ch[1] to ch[n]
 * represents a sentence of the above expression language.
 *
 * Recall that the lexical analysis of ch[1:n] may be done by a DFA. 
 * The program below does the lexical analysis concurrently.
 */


#define MAXP 5 /* maximum number of parallel threads */
 
enum token { NOTHING, ID, NUM, PLUS, TIMES, BLANK };
 
thread P (int i) {

  if (ch[i] == '+') result[i+1] = PLUS;
  else if (ch[i] == '*') result[i+1] = TIMES;
  else if (ch[i] == ' ') result[i+1] = BLANK;
  else if (ch[i] <= '9' && ch[i] >= '0') {
    await(result[i] != NOTHING);
    if (result[i] >= NUM) 
      result[i+1] = NUM;
    else
      result[i+1] = ID;
  } 
  else if ((ch[i] <= 'z' && ch[i] >= 'a') || 
           (ch[i] <= 'Z' && ch[i] >= 'A')) {
    result[i+1] = ID;
  }
}

main () {
  int i, j;
  char* ch = read_string();
  enum token * result = new enum token (strlen(ch)+2);
  init(result, NOTHING);
  result[0] = BLANK;
  i = 1;
  while (i < strlen(ch)) {
    j = 0;
    while (j < MAXP && i+j < strlen(ch)) {
      run P(i+j); j++;
    }
    wait(); /* wait that threads end */
    i += j;
  }

}
