(*  Press 
    Copyright (C) 2005, 2006 M. Sighireanu and L. Allali

    This file is part of Press.

    Press is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Press is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Press; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*)

(*
 *  Main entry.
 *
 *  File:         $RCSfile: prs_reach.ml,v $
 *  Author:       M. Sighireanu
 *  Version:      $Revision: 1.5 $
 *  Date:         $Date: 2006/05/31 19:44:34 $
 *)

(* ******************************************************************
 * Import of modules.
 * ******************************************************************)

open Prs_reach_util
open Prs_reach_eq
open Prs_reach_0
open Prs_reach_s
open Prs_reach_p
open Prs_reach_l


(* ******************************************************************
 * Usage of main.
 * ******************************************************************)

let usage_msg = "Usage: "^Sys.argv.(0)^" [-v] filename";;

let filename = ref "" ;;

let options_spec = 
	[("-v", (Arg.Set option_debug), "verbose mode");
         ("-e", (Arg.Unit (fun () -> option_equiv := 0)), "reachability modulo term equality");
         ("-0", (Arg.Unit (fun () -> option_equiv := 1)), "reachability modulo idle neutrality");
         ("-s", (Arg.Unit (fun () -> option_equiv := 2)), "reachability modulo idle neutrality and seq associativity");
         ("-p", (Arg.Unit (fun () -> option_equiv := 3)), "reachability modulo ~|");
         ("-l", (Arg.Unit (fun () -> option_equiv := 4)), "reachability modulo ~l")
        ]
;;

let file_spec s = 
	begin
	filename := s
	end
;;

(* ******************************************************************
 * Intersection with input automata.
 * ******************************************************************)

let check_intersection a_prs named_a = 
	if (fst named_a) <> "init" then
	begin
	(print_message ("check intersection with automaton "^(fst named_a)));
	(let _ = (Automaton.clean (snd named_a)) in
	 let a_inter = (Automaton.inter (snd named_a) a_prs) and
	     a_name = "inter_"^(fst named_a) in
         let a_inter_s = (Automaton.clean a_inter) and
	     a_filename = (!filename_debug)^"_"^a_name^".aut.tbk" in
	 (print_debug (function () ->
	 	(SpecPRS.save_automaton a_inter_s a_name a_filename)));
	 if (Automaton.is_language_empty a_inter_s) then
		(print_message "=====> empty")
	 else
		(print_message ("=====> not empty (see file "^a_filename^")"))
   	)
        end
	else
		()
;;


(* ******************************************************************
 * Entry point.
 * ******************************************************************)

let main () = 
	  begin
	  (* (print_message "begin parsing args"); *)
	  (Arg.parse options_spec file_spec usage_msg);
	  (* (print_message "parsing args OK"); *)
	  (if (String.length (!filename)) = 0 then
	   begin (Arg.usage options_spec usage_msg); 
		 raise (Arg.Bad "bad arguments")
	   end
	   else if (!option_debug) then
		filename_debug := "output/"^
					(Filename.chop_extension
					     (Filename.basename !filename))^
					"_reach_"^
					(if (!option_equiv = 0) then "eq"
					 else if (!option_equiv = 1) then "0"
					 else if (!option_equiv = 2) then "s"
					 else if (!option_equiv = 3) then "p"
					 else "l"
					 )
	   else ()
	  );
	  (print_message "parse input file") ;
	  let spec  = SpecPRS.file_parse (!filename) in
	  let sigma = SpecPRS.get_alphabet spec in
	  let ltrs  = SpecPRS.get_list_trs spec in
          let laut  = SpecPRS.get_list_automata spec in
	  begin
	    check_alphabet sigma ;           (* check the alphabet *)
	    (let r  = (get_prs ltrs)         (* check and return the prs *)
             and a0 = (get_init_aut laut)    (* check and return initial automaton *)
	     in
	     let a_prs = (if (!option_equiv = 0) then
			      (build_aut_reach_eq a0 r)
			     else if (!option_equiv = 1) then
			      (build_aut_reach_0 a0 r)
			     else if (!option_equiv = 2) then
			      (build_aut_reach_s a0 r)
			     else if (!option_equiv = 3) then
			      (build_aut_reach_p a0 r)
			     else
			      (build_aut_reach_l a0 r)
			  )
	     in
	     let _ = (List.map (check_intersection a_prs) laut) 
	     in
	         (print_message "exit.")
	     )
	  end
	  end
;;

main ()
